--[[ *** Create Ace2 AddOn *** ]]
RaidSpy = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDebug-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
local L = AceLibrary("AceLocale-2.2"):new("RaidSpy")
local Tablet = AceLibrary("Tablet-2.0")

RaidSpy.hasIcon = "Interface\\Icons\\INV_Misc_Spyglass_03"
RaidSpy.hideMenuTitle = true
RaidSpy.hasNoColor = true
RaidSpy.tooltipHiddenWhenEmpty = true

--[[ *** Initialize the AddOn *** ]]
function RaidSpy:OnInitialize()
	--[[ Register a DB for saving ]]
	self:RegisterDB("RaidSpyDB")
	self:RegisterDefaults('profile', {
		checks = {
			item = true,
			dura = true,
			reag = true,
			resi = true,
			chatR = true,
			erroR = true,
			saveData = true,
		}
	})
	
	self:BuildOpts()
	self.OnMenuRequest = self.opts
  
  --[[ Register chat commands ]]    
  self:RegisterChatCommand(L["consolecommands"], self.opts)
  
  --[[ Tell user we loaded ]]
	self:Print(L["translator"])
end

--[[ *** Do stuff when enabled? *** ]]
function RaidSpy:OnEnable()
	self.saved = { }
	
	--[[ Register Events for example ]]
  self:RegisterEvent("CHAT_MSG_ADDON")
  self:RegisterEvent("RAID_ROSTER_UPDATE")
end

--[[ *** Do stuff when disabled? *** ]]
function RaidSpy:OnDisable()
  --[[ Unregister Events for example ]]
  self:UnregisterAllEvents()
end
--[[ *** FuBar Text *** ]]
function RaidSpy:OnTextUpdate()
    self:SetText(L["addonnameFu"])
end

--[[ *** FuBar Tooltip *** ]]
function RaidSpy:OnTooltipUpdate()
	Tablet:SetTitle(L["addonnameFu"])
	local cat = Tablet:AddCategory(
  	'columns', 1
  )
  
  if UnitInRaid("player") then
  	if (not self.raidMembers) then self:RAID_ROSTER_UPDATE() end
  	
  	if (self.db.profile.checks.saveData) then
  		if ((self.saved.itemResponses) or (self.saved.duraResponses)) then
  			cat:AddLine(
			    'text', "Spy Info (validity unknown ;)",
			    'justify', "CENTER"
			  )
			  
			  cat = Tablet:AddCategory(
			  	'columns', 5,
			  	'hideBlankLine', true
			  )
			  local infoData = { text = "Name", text2 = "Item", text3 = "", text4 = "Dura", text5 = "Broken" }
			  cat:AddLine(infoData)
			  
			  for playername, _ in pairs(self.raidMembers) do
					local infoData = { text = "", text2 = "", text3 = "", text4 = "", text5 = "", text6 = "" }
					local isInteressting = false
				
				 	infoData["text"] = playername
				 	if (self.saved.itemResponses) then
					 	if (self.saved.itemResponses[playername]) then
					 		infoData["text2"] = self.saved.itemResponses[playername]
					 		isInteressting = true
					 	end
				 	end
				 	
				 	if (self.saved.duraResponses) then
					 	if (self.saved.duraResponses[playername]) then
					 		local p = math.floor(self.saved.duraResponses[playername]["cur"] / self.saved.duraResponses[playername]["max"] * 100)
					 		infoData["text4"] = p.."%"
					 		infoData["text5"] = self.saved.duraResponses[playername]["broken"]
					 		isInteressting = true
					 	end
				 	end
				 	
				 	if (isInteressting) then cat:AddLine(infoData) end
				end
			else
				cat:AddLine(
			    'text', "Spy has no info (yet)",
			    'justify', "CENTER"
			  )
			end
		else
  		cat:AddLine(
		    'text', "Advanced Spy features is turned off",
		    'justify', "CENTER"
		  )
		end
	else
		cat:AddLine(
	    'text', "You are not in a raid",
	    'justify', "CENTER"
	  )
	end
  
  
  Tablet:SetHint(L["tabletHint"])
end

--[[ *** EVENTS *** ]]
function RaidSpy:CHAT_MSG_ADDON()
	if ( arg1 == "CTRA" and arg3 == "RAID" ) then
		local msg = string.gsub(arg2, "%$", "s");
		msg = string.gsub(msg, "", "S");
		if (strsub(msg, strlen(msg)-7) == " ...hic!") then
			msg = strsub(msg, 1, strlen(msg)-8);
		end
		
		if (string.find(msg, "#")) then
			for k, v in pairs(self:Split(msg, "#")) do
				self:CheckEvent(arg4, v)
			end
		else
			self:CheckEvent(arg4, msg)
		end
	end
end

function RaidSpy:RAID_ROSTER_UPDATE()
	self.raidMembers = nil
	self.raidMembers = { }
	if (not self.db.profile.checks.saveData) then return end
	
	for i=1, GetNumRaidMembers() do
		self.raidMembers[(UnitName("raid"..i))] = true
	end
end

--[[ *** Functions *** ]]
function RaidSpy:CheckEvent(nick, msg)
	if (msg == "RSTC" and self.db.profile.checks.resi) then -- Resists
		self:Report(nick, L["ResistanceText"])
	elseif (msg == "REAC" and self.db.profile.checks.reag) then -- Reagents
		self:Report(nick, L["ReagentsText"])
	elseif (msg == "DURC" and self.db.profile.checks.dura) then --Durabillity
		self:Report(nick, L["DurabilityText"])
	elseif (string.find(msg, "^ITMC ") and self.db.profile.checks.item) then -- Items
		local _, _, itemName = string.find(msg, "^ITMC (.+)$")
		if (itemName) then
			self:Report(nick, (string.gsub(L["ItemText"], "{i}", itemName)))
		end
	end
	
	--[[ Tihi ]]
	if (not self.saved) then self.saved = { } end
	if (not self.db.profile.checks.saveData) then return end
	-- "ITM "..numitems.." "..itemname.." "..author
	if (string.find(msg, "^ITM ")) then
		if (not self.saved.itemResponses) then self.saved.itemResponses = { } end
		local _,_,numitems,itemname,requestby = string.find(msg, "^ITM ([-%d]+) (.+) ([^%s]+)$")
		self.saved.itemResponses[nick] = numitems.."x"..itemname
	end
	
	-- string.format("DUR %s %s %s %s", cur, max, broken, author)
	if (string.find(msg, "^DUR ")) then
		if (not self.saved.duraResponses) then self.saved.duraResponses = { } end
		local _,_,cur,max,broken,requestby = string.find(msg, "^DUR (%d+) (%d+) (%d+) ([^%s]+)$")
		self.saved.duraResponses[nick] = { }
		self.saved.duraResponses[nick]["cur"] = cur
		self.saved.duraResponses[nick]["max"] = max
		self.saved.duraResponses[nick]["broken"] = broken
	end
end

function RaidSpy:Report(nick, msg)
	if ((nick == UnitName("player")) and (not self.debugging)) then return end
	if (self.db.profile.checks.chatR) then
		self:Print(nick.." "..msg)
	end
	if (self.db.profile.checks.erroR) then
		UIErrorsFrame:AddMessage(L["AddOnErrorMessagePrefix"]..nick.." "..msg)
	end
end

function RaidSpy:Split(text, delimiter)
	local list = {}
  local pos = 1
  if strfind("", delimiter, 1) then -- this would result in endless loops
    error("delimiter matches empty string!")
  end
  while 1 do
    local first, last = strfind(text, delimiter, pos)
    if first then -- found?
      tinsert(list, strsub(text, pos, first-1))
      pos = last+1
    else
      tinsert(list, strsub(text, pos))
      break
    end
  end
  return list
end

function RaidSpy:PerformItemCheck(item)
	if (not UnitInRaid("player")) then
		self:Print(L["UnitNotInRaid"])
		return
	end
	if (IsAddOnLoaded("oRA2")) then
		-- IsRaidOfficer() -- IsRaidLeader()
		oRALItem:PerformItemCheck(item)
	elseif (IsAddOnLoaded("CT_RaidAssist")) then
		SlashCmdList["RAITEM"](item)
	else
		self:Print(L["cantdoitemcheck"])
	end
end

function RaidSpy:BuildOpts()
	self.opts = {
	  type = "group",
	  args = {
	  	headerName = {
				type = "header",
				name = L["addonname"],
				icon = self.hasIcon, --"Interface\\Icons\\INV_Misc_Spyglass_03",
				iconHeight = 16,
				iconWidth = 16,
				order = 1,
			},
			headerAuthor = {
				type = "header",
				name = "|cffffffff"..L["author"].."|r",
				textR = 1, 
				textG = 1, 
				textB = 1,
				order = 2,
			},
			headerVersion = {
				type = "header",
				name = "|cffffffff"..L["Version: $"]..self.version.."|r",
				order = 3,
			},
			headerspacer = {
				type = "header",
				order = 4,
			},
			typespacer = {
				type = "header",
				order = 6,
			},
			checksspacer = {
				type = "header",
				order = 8,
			},
			checks = {
				type = "group",
				name = L["predefinedChecks_name"],
				desc = L["predefinedChecks_desc"],
				order = 9,
				args = {
					std = {
						type = "group",
						name = L["predefinedChecks_std_name"],
						desc = L["predefinedChecks_std_desc"],
						order = 1,
						args = { },
					},
					zg = {
						type = "group",
						name = L["predefinedChecks_zg_name"],
						desc = L["predefinedChecks_zg_desc"],
						order = 2,
						args = { },
					},
					mc = {
						type = "group",
						name = L["predefinedChecks_mc_name"],
						desc = L["predefinedChecks_mc_desc"],
						order = 2,
						args = { },
					},
					bwl = {
						type = "group",
						name = L["predefinedChecks_bwl_name"],
						desc = L["predefinedChecks_bwl_desc"],
						order = 2,
						args = { },
					},
					aq = {
						type = "group",
						name = L["predefinedChecks_aq_name"],
						desc = L["predefinedChecks_aq_desc"],
						order = 2,
						args = { },
					},
					naxx = {
						type = "group",
						name = L["predefinedChecks_naxx_name"],
						desc = L["predefinedChecks_naxx_desc"],
						order = 2,
						args = { },
					},
				},
			},
	  },
	}
	
	for k,v in pairs(self.db.profile.checks) do
		self.opts.args[L[k.."_cmd"]] = {
			type = "toggle",
      name = L[k.."_name"],
      desc = L[k.."_desc"],
      get = function()
        return self.db.profile.checks[k]
      end,
      set = function()
        self.db.profile.checks[k] = not self.db.profile.checks[k]
      end,
      order = tonumber(L[k.."_order"]),
		}
	end
	
	self.opts.args.checks.args[L["Hearthstone_name"]] = {
		type = "execute",
		name = "["..L["Hearthstone_name"].."]",
		desc = L["Hearthstone_desc"],
	  func = function() self:PerformItemCheck(L["Hearthstone_name"]) end,
	}
	
	for k,_ in pairs(L["predefinedCheckItems"]) do
		for _,v in pairs(L["predefinedCheckItems"][k]) do
			local orderNum = 1
			local itemName = v
			if (string.find(itemName, "|")) then
				orderNum = tonumber(self:Split(itemName, "|")[1])
				itemName = self:Split(itemName, "|")[2]
			end
			self.opts.args.checks.args[k].args[string.gsub(itemName, "[^A-Za-z1-9_]", "")] = {
				type = "execute",
				name = "["..itemName.."]",
				desc = L["Run a check for: "]..itemName,
			  func = function() self:PerformItemCheck(itemName) end,
			  order = orderNum
			}
		end
	end
	
	return self.opts
end